from EVECelery.tasks.BaseTasks.ESIResqust import ESIRequest
from EVECelery.exceptions.tasks import InputValidationError


class CategoryInfo(ESIRequest):
    def get_key(self, category_id: int):
        return f"CategoryInfo-{category_id}"

    def route(self, category_id: int):
        return f"/universe/categories/{category_id}"

    def validate_inputs(self, category_id: int) -> None:
        try:
            int(category_id)
        except ValueError:
            raise InputValidationError("Input parameter must be an integer.")




from EVECelery.tasks.BaseTasks.ESIResqust import ESIRequest
from EVECelery.exceptions.tasks import InputValidationError
from .RegionInfo import RegionInfo


class ConstellationInfo(ESIRequest):
    def get_key(self, constellation_id: int):
        return f"ConstellationInfo-{constellation_id}"

    def route(self, constellation_id: int):
        return f"/universe/constellations/{constellation_id}"

    def _hook_after_esi_success(self, esi_response: dict) -> None:
        region_id = esi_response.get("region_id")
        if region_id:
            RegionInfo().get_async(ignore_result=True, region_id=region_id)

    def validate_inputs(self, constellation_id: int) -> None:
        try:
            int(constellation_id)
        except ValueError:
            raise InputValidationError("Input parameter must be an integer.")





from EVECelery.tasks.BaseTasks.ESIResqust import ESIRequest
from EVECelery.exceptions.tasks import InputValidationError
from .ConstellationInfo import ConstellationInfo


class SystemInfo(ESIRequest):
    def get_key(self, system_id: int):
        return f"SystemInfo-{system_id}"

    def route(self, system_id: int):
        return f"/universe/systems/{system_id}"

    def _hook_after_esi_success(self, esi_response: dict) -> None:
        constellation_id = esi_response.get("constellation_id")
        if constellation_id:
            ConstellationInfo().get_async(ignore_result=True, constellation_id=constellation_id)

    def validate_inputs(self, system_id: int) -> None:
        try:
            int(system_id)
        except ValueError:
            raise InputValidationError("Input parameter must be an integer.")





import os
import platform
import requests
from EVECelery.exceptions.utils import MissingHeaderEmail
from EVECelery import __version__


class RequestHeaders(object):
    @classmethod
    def get_email(cls):
        email = os.environ.get('EVECelery_Email')
        if email is None:
            raise MissingHeaderEmail("Missing 'header_email' variable. "
                                     "Please set this variable to your email address so ESI and ZK APIs "
                                     "can contact you if there are problems. "
                                     "Requests to ESI or ZK will not occur until this field is set.")
        else:
            return email

    @classmethod
    def get_headers(cls) -> dict:
        h = {"Accept":      "application/json",
             "From":        cls.get_email(),
             "Maintainer":  "maintainers@eveinsight.net",
             "User-Agent": f"EVECelery {__version__} (https://github.com/NatEVETools/EVECelery)"
                           f"Python/{platform.python_version()} "
                           f"Requests/{requests.__version__}"}
        return h



class BaseClient(object):
    def check_connection(self) -> bool:
        """
        Check that a connection is successful, else raise exception
        """
        raise NotImplementedError

    @property
    def connection_str(self) -> str:
        """
        Returns the connection string for use by Celery
        """
        raise NotImplementedError



from .CategoryInfo import CategoryInfo
from .ConstellationInfo import ConstellationInfo
from .FactionsList import FactionsList
from .GroupInfo import GroupInfo
from .RegionInfo import RegionInfo
from .SystemInfo import SystemInfo
from .TypeInfo import TypeInfo



from celery import Task
import redis
from EVECelery.clients.ClientRedis import ClientRedisLocks, ClientRedisCache


class BaseTask(Task):
    _client_redis_locks: ClientRedisLocks = None
    _client_redis_cache: ClientRedisCache = None

    def __init__(self):
        self.name = self.__class__.__name__

    @property
    def redis_locks(self) -> redis.Redis:
        if self._client_redis_locks is None:
            c = ClientRedisLocks()
            c.check_connection()
            self._client_redis_locks = c
        return self._client_redis_locks.redis

    @property
    def redis_cache(self) -> redis.Redis:
        if self._client_redis_cache is None:
            c = ClientRedisCache()
            c.check_connection()
            self._client_redis_cache = c
        return self._client_redis_cache.redis



__author_email__ = 'maintainers@eveinsight.net'
__description__ = 'ESI celery task queues for distributed API access using RabbitMQ and Redis.'
__license__ = 'GNU General Public License v3.0'
__title__ = 'EVECelery'
__url__ = 'https://github.com/NatEVETools/EVECelery'
__version__ = '0.14'



from celery import Celery, Task
from EVECelery.__version__ import __version__, __url__, __license__
from EVECelery.tasks.Alliance import *
from EVECelery.tasks.Character import *
from EVECelery.tasks.Corporation import *
from EVECelery.tasks.Market import *
from EVECelery.tasks.Routes import *
from EVECelery.tasks.Universe import *
from EVECelery.clients.ClientRabbitMQ import ClientRabbitMQ
from EVECelery.clients.ClientRedis import ClientRedisResultBackend
import os
from typing import Optional


class CeleryWorker(object):
    """Celery worker server wrapper.
    Creating an instance of this class creates a celery app and registers the default tasks.

    :param broker_user: RabbitMQ user
    :param broker_password: RabbitMQ password
    :param broker_host: RabbitMQ hostname
    :param broker_port: RabbitMQ port - normally 5672
    :param broker_vhost: RabbitMQ vhost - namespace for all EVECelery queues
    :param result_user: Redis user - normally "default" if not explicitly configured
    :param result_password: Redis password
    :param result_host: Redis hostname
    :param result_port: Redis port - normally 6379
    :param result_db: Redis db - normally 0 for the default db
    :param config_object: Custom config object to overwrite the default EVECelery.celeryconfig - optional
    :param esi_queue_prefix: Prefix to add to all generated ESI queue names
    """

    def __init__(self, broker_user: Optional[str] = None, broker_password: Optional[str] = None,
                 broker_host: Optional[str] = None, broker_port: Optional[int] = None,
                 broker_vhost: Optional[str] = None,
                 result_user: Optional[str] = None, result_password: Optional[str] = None,
                 result_host: Optional[str] = None, result_port: Optional[int] = None, result_db: Optional[int] = None,
                 config_object: str = "EVECelery.celeryconfig", esi_queue_prefix: str = "ESI-",
                 connection_check: bool = False):
        self.broker = ClientRabbitMQ(user=broker_user, password=broker_password, host=broker_host, port=broker_port,
                                     vhost=broker_vhost)
        self.result_backend = ClientRedisResultBackend(user=result_user, password=result_password, host=result_host,
                                                       port=result_port, db=result_db)

        if connection_check:
            self.result_backend.check_connection()

        self.max_concurrency = int(os.environ.get('EVECelery_MaxConcurrency', 4))

        self.esi_queue_prefix = esi_queue_prefix
        self.app = Celery("EVECelery")
        self.app.config_from_object(config_object)
        self.app.conf.update(broker_url=self.broker.connection_str)
        self.app.conf.update(result_backend=self.result_backend.connection_str)
        self.app.conf.update(task_default_queue=f"{self.esi_queue_prefix}Default")

        self.queues = [f"{self.esi_queue_prefix}Default"]
        self.task_routes = {}
        self.beat_schedule = {}
        self._register_defaults()

    def esi_tasks(self):
        """Default ESI tasks

        :return: ESI task instances registered by default
        """
        yield AllianceInfo()
        yield CharacterPublicInfo()
        yield CorporationInfo()
        yield PricesList()
        yield CategoryInfo()
        yield ConstellationInfo()
        yield FactionsList()
        yield GroupInfo()
        yield RegionInfo()
        yield Route()
        yield SystemInfo()
        yield TypeInfo()

    def tasks_to_register(self):
        """Yields tuple pairs of (task, queue name) to register with the Celery app

        :return: yields tuple consisting of (task instance, queue name)
        """
        for t in self.esi_tasks():
            yield (t, f"{self.esi_queue_prefix}{t.name}")

    def _register_defaults(self):
        for t in self.tasks_to_register():
            self.register_task(t[0])
            self.register_additional_queue(t[1])
            self.register_task_route(t[0].name, t[1])

    def register_additional_queue(self, queue: str):
        """Register an additional queue that this Celery app should process.

        :param queue: Name of the queue
        :return: None
        """
        self.queues.append(queue)

    def register_task_route(self, task_name: str, queue_name: str):
        """Register a task to a specific queue using a Celery task route.

        :param task_name: The name of the task
        :param queue_name: Name of the queue to route tasks to.
        :return: None
        """
        self.task_routes[task_name] = {"queue": queue_name}
        self.app.conf.update(task_routes=self.task_routes)

    def register_task(self, task: Task):
        """Register a task instance with the Celery app. A task must inherit from the Celery Task base class.

        :param task: A task instance
        :return: None
        """
        self.app.register_task(task)

    @classmethod
    def print_header(cls):
        print(f"EVECelery {__version__} ({__url__})\n{__license__}")

    def start(self):
        """Starts the Celery app and beings processing messages in the queues.

        :return: None
        """
        self.print_header()
        self.app.start(argv=["worker", "-l", "WARNING", f"--autoscale={self.max_concurrency},1",
                             "-Q", ",".join(self.queues)])


class CeleryBeat(CeleryWorker):
    """Celery beat scheduler for periodic tasks.

    """

    def schedule_task(self, schedule_name: str, schedule_config: dict):
        """Schedule a task to run at intervals. The passed in schedule object is a dictionary following the
        format and fields described here:
        https://docs.celeryproject.org/en/stable/userguide/periodic-tasks.html#available-fields

        :param schedule_name: Name of the scheduled job. This must be unique.
        :param schedule_config: The scheduled job config as specified at
            https://docs.celeryproject.org/en/stable/userguide/periodic-tasks.html#available-fields
        :return: None
        """
        self.beat_schedule[schedule_name] = schedule_config
        self.app.conf.update(beat_schedule=self.beat_schedule)

    def start(self):
        self.print_header()
        self.app.start(argv=["beat"])



import os
import sys
from typing import Optional
from pydantic import BaseModel
import traceback
from .BaseClient import BaseClient


class ConfigRabbitMQ(BaseModel):
    user: str
    password: str
    host: str
    port: int
    vhost: str


class ClientRabbitMQ(BaseClient):
    """

    :param user: RabbitMQ user
    :param password: RabbitMQ password
    :param host: RabbitMQ hostname
    :param port: RabbitMQ port - normally 5672
    :param vhost: RabbitMQ vhost - namespace for all queues
    """

    def __init__(self, user: Optional[str] = None, password: Optional[str] = None, host: Optional[str] = None,
                 port: Optional[int] = None, vhost: Optional[str] = None):
        try:
            if user is None:
                user = os.environ['EVECelery_RabbitMQ_User']
            if password is None:
                password = os.environ['EVECelery_RabbitMQ_Password']
            if host is None:
                host = os.environ['EVECelery_RabbitMQ_Host']
            if port is None:
                port = os.environ.get('EVECelery_RabbitMQ_Port', 5672)
            if vhost is None:
                vhost = os.environ['EVECelery_RabbitMQ_Vhost']
        except KeyError as ex:
            traceback.print_exc()
            print(f'The environmental variable is not set for {ex}. '
                  f'Please set this env var or pass it as an argument to initialize ClientRabbitMQ.')
            sys.exit(1)
        self.config = ConfigRabbitMQ(user=user, password=password, host=host, port=port, vhost=vhost)

    @property
    def connection_str(self):
        return f"amqp://{self.config.user}:{self.config.password}@{self.config.host}:{self.config.port}/{self.config.vhost}"



from EVECelery.tasks.BaseTasks.ESIResqust import ESIRequest
from EVECelery.exceptions.tasks import InputValidationError


class CharacterPublicInfo(ESIRequest):
    def ttl_404(self) -> int:
        return 86400  # current esi x-cached-seconds header

    def get_key(self, character_id: int):
        return f"CharacterPublicInfo-{character_id}"

    def route(self, character_id: int):
        return f"/characters/{character_id}"

    def validate_inputs(self, character_id: int) -> None:
        try:
            int(character_id)
        except ValueError:
            raise InputValidationError("Input parameter must be an integer.")




import os
import sys
from typing import Optional
from pydantic import BaseModel, validator
import traceback
import redis


class ConfigRedis(BaseModel):
    user: str
    password: str
    host: str
    port: int
    db: int

    @validator('db')
    def valid_db_number(cls, v):
        if not 0 <= v <= 15:
            raise ValueError('Redis DB must be between 0 and 15.')
        return v


class ClientRedis:
    """

    :param user: Redis user
    :param password: Redis password
    :param host: Redis hostname
    :param port: Redis port - normally 6379
    :param db: Redis DB - namespace for all queues
    """

    def __init__(self, user: str, password: str, host: str, port: int, db: int):
        self.config = ConfigRedis(user=user, password=password, host=host, port=port, db=db)
        self._redis = redis.Redis(username=self.config.user,
                                  host=self.config.host,
                                  port=self.config.port,
                                  db=self.config.db,
                                  password=self.config.password)

    def check_connection(self) -> bool:
        return self.redis.ping()

    @property
    def redis(self) -> redis.Redis:
        return self._redis

    @property
    def connection_str(self):
        return f"redis://{self.config.user}:{self.config.password}@{self.config.host}:{self.config.port}/{self.config.db}"


class ClientRedisResultBackend(ClientRedis):
    def __init__(self, user: Optional[str] = None, password: Optional[str] = None, host: Optional[str] = None,
                 port: Optional[int] = None, db: Optional[int] = None):
        try:
            if user is None:
                user = os.environ['EVECelery_Redis_ResultBackend_User']
            if password is None:
                password = os.environ['EVECelery_Redis_ResultBackend_Password']
            if host is None:
                host = os.environ['EVECelery_Redis_ResultBackend_Host']
            if port is None:
                port = os.environ.get('EVECelery_Redis_ResultBackend_Port', 6379)
            if db is None:
                db = os.environ.get('EVECelery_Redis_ResultBackend_DB', 0)
        except KeyError as ex:
            traceback.print_exc()
            print(f'The environmental variable is not set for {ex}. '
                  f'Please set this env var or pass it as an argument to initialize ClientRedisResultBackend.')
            sys.exit(1)
        super().__init__(user=user, password=password, host=host, port=port, db=db)


class ClientRedisLocks(ClientRedis):
    def __init__(self, user: Optional[str] = None, password: Optional[str] = None, host: Optional[str] = None,
                 port: Optional[int] = None, db: Optional[int] = None):
        try:
            if user is None:
                user = os.environ.get('EVECelery_Redis_Locks_User',
                                      os.environ.get('EVECelery_Redis_ResultBackend_User'))
                if user is None:
                    raise KeyError('EVECelery_Redis_Locks_User')
            if password is None:
                password = os.environ.get('EVECelery_Redis_Locks_Password',
                                          os.environ.get('EVECelery_Redis_ResultBackend_Password'))
                if password is None:
                    raise KeyError('EVECelery_Redis_Locks_Password')
            if host is None:
                host = os.environ.get('EVECelery_Redis_Locks_Host',
                                      os.environ.get('EVECelery_Redis_ResultBackend_Host'))
                if host is None:
                    raise KeyError('EVECelery_Redis_Locks_Host')
            if port is None:
                port = os.environ.get('EVECelery_Redis_Locks_Port',
                                      os.environ.get('EVECelery_Redis_ResultBackend_Port', 6379))
            if db is None:
                db = os.environ.get('EVECelery_Redis_Locks_DB',
                                    int(os.environ.get('EVECelery_Redis_ResultBackend_DB', 0)) + 1)
        except KeyError as ex:
            traceback.print_exc()
            print(f'The environmental variable is not set for {ex}. '
                  f'Please set this env var or pass it as an argument to initialize ClientRedisLocks.')
            sys.exit(1)
        super().__init__(user=user, password=password, host=host, port=port, db=db)


class ClientRedisCache(ClientRedis):
    def __init__(self, user: Optional[str] = None, password: Optional[str] = None, host: Optional[str] = None,
                 port: Optional[int] = None, db: Optional[int] = None):
        try:
            if user is None:
                user = os.environ.get('EVECelery_Redis_Cache_User',
                                      os.environ.get('EVECelery_Redis_ResultBackend_User'))
                if user is None:
                    raise KeyError('EVECelery_Redis_Cache_User')
            if password is None:
                password = os.environ.get('EVECelery_Redis_Cache_Password',
                                          os.environ.get('EVECelery_Redis_ResultBackend_Password'))
                if password is None:
                    raise KeyError('EVECelery_Redis_Cache_Password')
            if host is None:
                host = os.environ.get('EVECelery_Redis_Cache_Host',
                                      os.environ.get('EVECelery_Redis_ResultBackend_Host'))
                if host is None:
                    raise KeyError('EVECelery_Redis_Cache_Host')
            if port is None:
                port = os.environ.get('EVECelery_Redis_Cache_Port',
                                      os.environ.get('EVECelery_Redis_ResultBackend_Port', 6379))
            if db is None:
                db = os.environ.get('EVECelery_Redis_Cache_DB',
                                    int(os.environ.get('EVECelery_Redis_ResultBackend_DB', 0)) + 2)
        except KeyError as ex:
            traceback.print_exc()
            print(f'The environmental variable is not set for {ex}. '
                  f'Please set this env var or pass it as an argument to initialize ClientRedisCache.')
            sys.exit(1)
        super().__init__(user=user, password=password, host=host, port=port, db=db)



from EVECelery.tasks.BaseTasks.ESIResqust import ESIRequest


class FactionsList(ESIRequest):
    def get_key(self):
        return f"FactionsList"

    def route(self):
        return f"/universe/factions"

    def validate_inputs(self, **kwargs) -> None:
        return




from EVECelery.tasks.BaseTasks.ESIResqust import ESIRequest
from EVECelery.exceptions.tasks import InputValidationError


class RegionInfo(ESIRequest):
    def get_key(self, region_id: int):
        return f"RegionInfo-{region_id}"

    def route(self, region_id: int):
        return f"/universe/regions/{region_id}"

    def validate_inputs(self, region_id: int) -> None:
        try:
            int(region_id)
        except ValueError:
            raise InputValidationError("Input parameter must be an integer.")


